<?php

namespace Drupal\jsonapi_layout_enhancer\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Url;
use Drupal\jsonapi\Normalizer\HttpExceptionNormalizer;
use Drupal\node\NodeInterface;
use Drupal\path_alias\PathAliasInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Making page API calls dynamic.
 */
class GetJsonApiDataFromAliasController extends ControllerBase {

  /**
   * The request_stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config.factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The language_manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The path_alias storage.
   *
   * @var \Drupal\path_alias\PathAliasInterface
   */
  protected $pathAliasStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    LanguageManagerInterface $languageManager,
    ConfigFactoryInterface $configFactory,
    RequestStack $requestStack,
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->pathAliasStorage = $this->entityTypeManager->getStorage('path_alias');
    $this->languageManager = $languageManager;
    $this->configFactory = $configFactory;
    $this->requestStack = $requestStack;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Drupal service container.
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('language_manager'),
      $container->get('config.factory'),
      $container->get('request_stack'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function content($langcode, $alias, $alias_2) {
    $nodeStorage = $this->entityTypeManager->getStorage('node');
    $bundle = '';
    $uuid = '';
    $frontpage = '';
    $aliasObj = '';
    $sourceUrl = '';
    if ($alias == '') {
      $frontpage = $this->configFactory->get('system.site')->get('page.front');
    }
    else {
      $alias_value = '';
      if ($alias_2 != '') {
        $alias_value = '/' . $alias . '/' . $alias_2;
      }
      else {
        $alias_value = '/' . $alias;
      }
      $aliasObj =
        $this->pathAliasStorage->loadByProperties(
          ['alias' => $alias_value, 'langcode' => $langcode]);
    }

    if ((is_array($aliasObj) && count($aliasObj) > 0) || $frontpage != '') {
      if ($frontpage == '') {
        $sourceUrl = current($aliasObj);
      }

      if (($sourceUrl instanceof PathAliasInterface) || ($frontpage != '')) {
        if ($frontpage != '') {
          $sourceUrlExplode = explode('/', $frontpage);
        }
        else {
          $sourceUrlExplode = explode('/', $sourceUrl->getPath());
        }
        $nodeId = $sourceUrlExplode[2];

        $nodeObj = $nodeStorage->load($nodeId);

        if ($nodeObj instanceof NodeInterface) {
          $uuid = $nodeObj->get('uuid')->value;
          $bundle = $nodeObj->bundle();
        }
      }
    }
    else {
      $error = [
        'status' => '404',
        'title' => 'Not Found',
        'detail' => 'The requested path alias is not available.',
        'links' => [
          'via' => ['href' => "/jsonapi/page/$langcode/$alias"],
          'info' => ['href' => HttpExceptionNormalizer::getInfoUrl(404)],
        ],
      ];
      return new JsonResponse(['errors' => [$error]], 404);
    }
    $language = $this->languageManager->getLanguage($langcode);
    $url = Url::fromUri('internal:' . '/jsonapi/node/' . $bundle . '/' . $uuid, ['language' => $language]);
    $resourceVersion = $this->requestStack->getCurrentRequest()->query->get('resourceVersion');
    if (isset($resourceVersion) && !empty($resourceVersion)) {
      $url->setOptions(['query' => ['resourceVersion' => $resourceVersion]]);
    }
    $ret = $url->toString();
    return new RedirectResponse($ret);
  }

}
