<?php

namespace Drupal\jsonapi_layout_enhancer\Plugin\jsonapi\FieldEnhancer;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\jsonapi\ResourceType\ResourceTypeRepositoryInterface;
use Drupal\jsonapi_extras\EntityToJsonApi;
use Drupal\jsonapi_extras\Plugin\ResourceFieldEnhancerBase;
use Shaper\Util\Context;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Resource field enhancer for block content.
 *
 * @ResourceFieldEnhancer(
 *   id = "block_content",
 *   label = @Translation("Custom block content and attributes"),
 *   description = @Translation("Custom block content and attribute")
 * )
 */
class LayoutBlockContentEnhancer extends ResourceFieldEnhancerBase implements ContainerFactoryPluginInterface {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  public EntityTypeManagerInterface $entityTypeManager;

  /**
   * JSON:API resource type repository.
   *
   * @var \Drupal\jsonapi\ResourceType\ResourceTypeRepositoryInterface
   */
  public ResourceTypeRepositoryInterface $resourceTypeRepository;

  /**
   * JSON:API Extras entity to json:api.
   *
   * @var \Drupal\jsonapi_extras\EntityToJsonApi
   */
  public EntityToJsonApi $entityToJsonApi;

  /**
   * Contructor.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ResourceTypeRepositoryInterface $resource_type_repository,
    EntityToJsonApi $entity_to_json_api,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->resourceTypeRepository = $resource_type_repository;
    $this->entityToJsonApi = $entity_to_json_api;
  }

  /**
   * Container.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('jsonapi.resource_type.repository'),
      $container->get('jsonapi_extras.entity.to_jsonapi')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function doUndoTransform($data, Context $context) {
    foreach ($data['components'] as &$component) {
      if (!isset($component['configuration']['provider'])) {
        continue;
      }
      switch ($component['configuration']['provider']) {
        case 'block_content':
          $this->doTransformContentBlock($component, $context);
          break;

        default:
          // Nothing to do.
          break;

      }
    }
    return $data;
  }

  /**
   * Content block transformer.
   *
   * @param array $component
   *   Internals of Layout builder structure.
   * @param \Shaper\Util\Context $context
   *   Context object.
   */
  protected function doTransformContentBlock(array &$component, Context $context) {
    $entity_type = $component['configuration']['provider'];
    $uuid = explode(':', $component['configuration']['id'])[1];
    $blocks = $this->entityTypeManager->getStorage($entity_type)->loadByProperties(['uuid' => $uuid]);
    $block = reset($blocks);
    $component['configuration']['uuid'] = $uuid;
    if (!empty($block)) {
      $component['configuration']['type'] = $block->get('type')->getString();
      $block_data = $this->entityToJsonApi->normalize($block);
      $component['configuration']['block_content_data'] = $block_data['data'];
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function doTransform($data, Context $context) {
    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function getOutputJsonSchema() {
    $component_props = [
      'uuid' => ['type' => 'string'],
      'region' => ['type' => 'string'],
      'configuration' => ['type' => 'object'],
    ];
    $component_schema = ['type' => 'object', 'properties' => $component_props];

    return [
      'oneOf' => [
        ['type' => 'null'],
        [
          'type' => 'object',
          'properties' => [
            'layout_id' => ['type' => 'string'],
            'components' => ['type' => 'array', 'items' => $component_schema],
          ],
        ],
      ],
    ];
  }

}
